% IGEM INSA-UPS Toulouse 2022 - DAISY team
% link to wiki : https://2022.igem.wiki/toulouse-insa-ups/index.html
% link to model : https://2022.igem.wiki/toulouse-insa-ups/model
% Main script of Agregation Dynamics, MATLAB
% Adapted from a model used on : 
% FRANCES C, LINE A. Comminution Process Modeling Based on the Monovariate
% and Bivariate Direct Quadrature Method of Moments AIChE J. 2014 DOI:10.1002/aic.14358
% DQMOM Bivariate - Case solving of homogenous system with two properties
% for biologic phase.
% SimulationX call a ode function for solving a differential equation
% system characterizing the transport (derivate only from time) of weights
% and abscissas.
% This program uses the following functions : 
% DQMOM_Solver : Solver of linear equation system with A*Alpha=PBE_Model
% PBE_Model : Sink term - right hand side of the linear equation system

clear all
close all

% Gestion of folders
addpath(genpath('C:\Users\Raphal\Desktop\Matlab\Modlisation\PBE_Modeling_V8\Model'))
addpath(genpath('C:\Users\Raphal\Desktop\Matlab\Modlisation\PBE_Modeling_V8\Results'))

% Beginning of the timespan for calculating
tic

% Global variables, used on both program
global N vK vL  
global G K_beta K_gamma

% Choice of the number of nodes (N=2 or N=3)
N = 2;

% Initial conditions of weigths and abscissas for N=2 or N=3
if N==2
    % Weights ans abscissas expression, then build of alpha0 matrix contianing
    % each properties of the biologic phase on initial state. 
    % 1 and 2 are index of two abscissas and associated weights.
    % L1 represents the number of A on an aggregate
    % L2 represents the number of D on an aggregate
    alpha0=zeros(1,3*N);
    w1 = 0.99; 
    L11 = 1;
    L21 = 1;
    w2 = (1-w1)/2;
    L12 = 2;
    L22 = 2;
    alpha0=[w1 w2 w1*L11 w2*L12 w1*L21 w2*L22];
else    % N=3;
    % Weights and abscissas expression, then build of alpha0 matrix contianing
    % each properties of the biologic phase on initial state. 
    % 1,2 and 3 are index of two abscissas and associated weights.
    % L1 represents the number of A on an aggregate
    % L2 represents the number of D on an aggregate
    alpha0=zeros(1,3*N);
    w1 = 0.8;
    w2 = 0.1; 
    w3 = 1-w1-w2;
    L11 = 1;
    L21 = 1;
    L12 = 100;
    L22 = 50;
    L13 = 50;
    L23 = 100;
    alpha0=[w1 w2 w3 w1*L11 w2*L12 w3*L13 w1*L21 w2*L22 w3*L23];
end

% Definition of mixed moments, vK contains considered on the first
% property (Percentage of A on aggregate), vL the second property
% (Percentage of D on aggregate). 

% Different choices of couples possible for N=2
% Case1
% vK=[0 1/3 2/3 0 0 1/3];
% vL=[0 0 0 1/3 2/3 1/3];
% Case2
% vK=[0 1 0 1 2 1];
% vL=[0 0 1 1 0 2];
% Case3 
% vK=[0 1 0 2 0 3];
% vL=[0 0 1 0 2 0];
% Case4, FOX 2006, showed problems of mass conservation
% vK=[0 1/3 2/3 1 0 0];
% vL=[0 0 0 0 1/3 2/3];

% Different choices of couples possible for N==3
% Case1
% vK=[0 1 2 0 1 0 2 0 3];
% vL=[0 1 2 1 0 2 0 3 0];
%Case2
% vK=[0 1/3 2/3 0 1/3 0 2/3 0 1];
% vL=[0 1/3 2/3 1/3 0 2/3 0 1 0];
%Case3
% vK=[0 1 2 0 1 2 0 2 1];
% vL=[0 1 2 1 0 0 2 1 2];
%Case4
% vK=[0 1/3 2/3 1 4/3 5/3 0 0 0];
% vL=[0 0 0 0 0 0 1/3 2/3 1];
%Case5
% vK=[0 1/3 2/3 0 1/3 1/3 2/3 2/3 1];
% vL=[0 1/3 2/3 1/3 0 2/3 1/3 1 2/3];

if N ==2 %Case3
%     vK=[0 1/3 2/3 0 0 1/3];
%     vL=[0 0 0 1/3 2/3 1/3];
vK=[0 1 0 1 2 1];
vL=[0 0 1 1 0 2];
%     vK=[0 1 0 2 0 3];
%     vL=[0 0 1 0 2 0];
%     vK=[0 1/3 2/3 1 0 0];
%     vL=[0 0 0 0 1/3 2/3];

else % Case3
    vK=[0 1 2 0 1 2 0 2 1];
    vL=[0 1 2 1 0 0 2 1 2];
end

% Data for aggregation
G=20;       % speed gradient in 1/s
K_beta=0.01;   % Constant for aggregation kernel
K_gamma=1e4;  % Constant linked to A and D sites

% ODE solving
tini=0;
tfin=3600;
[T,alpha]=ode23tb(@DQMOM_Solver,[tini tfin],alpha0);



% Expression of abscissas for each properties
taille=size(alpha);
L1=zeros(taille(1),N);
L2=zeros(taille(1),N);
for i=1:N
    L1(:,i)=alpha(:,N+i)./alpha(:,i);
    L2(:,i)=alpha(:,2*N+i)./alpha(:,i);
end

% Plotting
if N == 2 % N = 2
    figure(1) 
    hold on
    plot(T,L1(:,1),'<k',T,L1(:,2),'>k') 
    title('Variation of number of strain A')
    legend('A1','A2') 
    xlabel('Temps(min)')
    ylabel('Abscisses - A')
    hold off
    figure(2) 
    hold on
    plot(T,L2(:,1),'<r',T,L2(:,2),'>r')
    title('Variation of number of strain B')
    legend('D1','D2')
    xlabel('Temps(min)'),ylabel('Abscisses - D')
    hold off
    figure(3) 
    hold on
    title('Evolution of percentage of A and D on aggregates')
    plot(T,alpha(:,1),'<k',T,alpha(:,2),'>k')
    legend('w1 (% of A)','w2 (% of D)')
    xlabel('Temps(min)'),ylabel('Poids ')
    hold off
 else   % N = 3
     figure(1)
     hold on
     plot(T,L1(:,1),'<k',T,L1(:,2),'ok',T,L1(:,3),'>k')
     title('Variation of the number of strain A')
     legend('A1','A2','A3')
     xlabel('Temps(min)'),ylabel('Abscisses - A')
     hold off
     figure(2)
     hold on
     plot(T,L2(:,1),'<r',T,L2(:,2),'or',T,L2(:,3),'>r')
     title('Variation of number of strain B')
     legend('D1','D2','D3')
     xlabel('Temps(min)'),ylabel('Abscisses - D')
     hold off
     figure(3)
     hold on
     plot(T,alpha(:,1),'<k',T,alpha(:,2),'ok',T,alpha(:,3),'>k')
     title('Evolution of percentage of A and D on aggregates')
     legend('w1','w2','w3')
     xlabel('Temps(min)'),ylabel('Poids') 
     hold off
end

% Calculs of moments
m00=zeros(taille(1),1);
m10=zeros(taille(1),1);
m01=zeros(taille(1),1);
m11=zeros(taille(1),1);
m_somme =  zeros(taille(1),1);
for j=1:taille(1)
    for i=1:N
        m00(j)=m00(j)+alpha(j,i);
        m10(j)=m10(j)+alpha(j,i)*L1(j,i);
        m01(j)=m01(j)+alpha(j,i)*L2(j,i);
        m11(j)=m11(j)+alpha(j,i)*L1(j,i)*L2(j,i);
    end
    m_somme(j) = m01(j)+m10(j);
end

% Plotting
figure(4)
hold on
plot(T,m00,'+',T,m10,'o',T,m01,'*',T,m11)
title('Graph of variation of moments')
legend('m00 DQMOM','m10 DQMOM','m01 DQMOM','m11 DQMOM')
xlabel('Temps(min)'),ylabel('moments')
hold off
figure(5)
semilogy(T,m00,'+',T,m10,'o',T,m01,'*',T,m11)
title('Graph of variation of moments with log scale')
legend('m00 DQMOM','m10 DQMOM','m01 DQMOM','m11 DQMOM')
xlabel('Temps(min)'),ylabel('moments')
figure(6)
hold on
plot(T,m_somme,'+')
title('Control graph with the sum of particles on the system')
xlabel('Temps(min)'), ylabel('moments')
hold off


% End of timespan
toc

% save C:\Users\Raphal\Desktop\Matlab\Modlisation\PBE_Modeling_V8\Results\simulation13 alpha0 N T alpha tfin



